﻿#Skill2.5 Configure RAS Gateway

#Implement L3 Gateway
$NCServiceName = “NC” # This is the name of the Network Controller Network Service in VMM
$HNVVMNetwork = “Tenant A Network” # Name of the HNV Tenant VM Network
$VLANVMNetwork = “Yellow Network” # Name of the VLAN-based VM Network
$LocalIP = @(“10.254.254.253/29”) # IP Address on local side for routing
$PeerIP = @(“10.254.254.254”) # IP Address on remote side for routing
$GWSubnet = “10.254.254.0/29” # VM Network Gateway subnet
$Routes = @(“172.16.10.0/24”) # Routes to for VPN connection

$gw = Get-SCNetworkGateway | ? Service -match $NCServiceName
$vmNet1 = Get-SCVMNetwork -Name $HNVVMNetwork
$vmNet2 = Get-SCVMNetwork -Name $VLANVMNetwork
$vmNetGW = Add-SCVMNetworkGateway -Name $($HNVVMNetwork + “_Gateway”) -EnableBGP $false -NetworkGateway $gw -VMNetwork $vmNet1 -RoutingIPSubnet $GWSubnet
$vpn = Add-SCVPNConnection -Name $($HNVVMNetwork + “_” + $VLANVMNetwork) -VMNetworkGateway $vmNetGW -Protocol L3 -NextHopNetwork $vmNet2 -LocalIPAddresses $LocalIP -PeerIPAddresses $PeerIP
$routes | % {Add-SCNetworkRoute -IPSubnet $_ -RunAsynchronously -VPNConnection $vpn -VMNetworkGateway $vmNetGW}

#Implement GRE Tunnel
(Get-NetworkControllerVirtualGateway -ConnectionUri $uri).properties.NetworkConnections.properties.SourceIPAddress

Add-VPNS2SInterface -GreTunnel -Name “Tun0” -GreKey 1234 -Destination <GRE VIP> -IPv4Subnet “<Virtual Subnet/Prefix>:100” -SourceIPAddress <Gateway Public IP>
Set-VPNS2SInterface -Name “Tun0” -GreTunnel -AdminStatus $true

#Implement S2S Tunnel
Add-VPNS2SInterface -Name “Tun1” -Protocol IKEv2 -AuthenticationMethod PSKOnly -SharedSecret “ABC” -IPv4Subnet “<Virtual Subnet/Prefix>:100” -Destination <GW Public VIP>

#Configure BGP
Add-BgpPeer -Name GW01 -LocalIPAddress 10.10.10.1 -PeerIPAddress 10.10.10.7 -LocalASN 65001 -PeerASN 65002 -OperationMode Mixed -PeeringMode Automatic

#Implement RAS Gateway using PowerShell
Add-WindowsFeature -Name RemoteAccess -IncludeAllSubFeature -IncludeManagementTools
Install-RemoteAccess -MultiTenancy

$credential = New-Object Microsoft.Windows.Networkcontroller.credentialproperties
$credential.type = “usernamePassword”
$credential.username = “Contoso\DNSAdmin”
$credential.password = “<Plain Text Password>”
New-NetworkControllerCredential -ConnectionUri $uri -Properties $credential -ResourceId “Gateway-Credential”

$vmguid = (get-childitem -Path "HKLM:\software\microsoft\virtual machine\guest" | get-itemproperty).virtualmachineid

$credential = Get-NetworkControllerCredential -ConnectionUri $uri -ResourceId “Gateway-Credential”
$server = New-Object Microsoft.Windows.NetworkController.VirtualServerProperties
$server.connections = @([Microsoft.Windows.NetworkController.Connection]@{ManagementAddresses=@("10.184.108.11");Credential=$credential})
$server.VMGuid = $vmguid
New-NetworkControllerVirtualServer -ConnectionUri $uri -Properties $server -ResourceId “RAS-Gateway-10-184-108-11”

$nicprops = New-Object New-Object Microsoft.Windows.NetworkController.NetworkInterfaceProperties
New-NetworkControllerNetworkInterface -ConnectionUri $uri -ResourceId “InternatlNic-RAS-Gateway-10-184-108-11”
$transitNetwork = Get-NetworkControllerLogicalNetwork -ConnectionUri $uri -ResourceId “Transit-Network”
$ip = New-Object Microsoft.Windows.NetworkController.NetworkInterfaceIpConfigurationProperties
$ip.PrivateAddress = “10.10.10.253”
$ip.Subnet = $transitNetwork
$nicprops.IPConfigurations = $ip
New-NetworkControllerNetworkInterface -ConnectionUri $uri -ResourceId “ExternalNic-RAS-Gateway-10-184-108-11”

$nics = New-Object Microsoft.Windows.NetworkController.NetworkInterfaces
$nics.InternalNetworkInterface = Get-NetworkControllerNetworkInterface -ConnectionUri $uri -ResourceId “InternatlNic-RAS-Gateway-10-184-108-11”
$nics.ExternalNetworkInterface = Get-NetworkControllerNetworkInterface -ConnectionUri $uri -ResourceId “ExternalNic-RAS-Gateway-10-184-108-11”
$bgp = New-Object Microsoft.Windows.NetworkController.GatewayBgpConfig
$bgp.ExtAsNumber = “0.65004”
$bgp.BgpPeer = [Microsoft.Windows.NetworkController.GatewayBgpPeer]@{PeerIP=”10.10.10.1”;PeerExtAsNumber=”0.65001”}
$server = Get-NetworkControllerVirtualServer -ConnectionUri $uri -ResourceId “RAS-Gateway-10-184-108-11”
$pool = Get-NetworkControllerGatewayPool -ConnectionUri $uri -ResourceId “Pool1”
$gateway = New-Object Microsoft.Windows.NetworkController.GatewayProperties
$gateway.Type = All
$gateway.TotalCapacity = 1024000
$gateway.BgpConfig = $bgp
$gateway.VirtualServer = $server
$gateway.NetworkInterfaces = $nics
$gateway.Pool = $pool
New-NetworkControllerGateway -ConnectionUri $uri -ResourceId “Gateway1” -Properties $gateway

#Manage RAS Gateway via Network Controller
$subnet = [Microsoft.Windows.NetworkController.VirtualSubnetProperties]@{AddressPrefix=”10.254.254.0/29”}
New-NetworkControllerVirtualSubnet -ConnectionUri $uri -ResourceId “Tenant1_VNet1_RoutingSubnet” -Properties $subnet -VirtualNetworkId "Tenant1_VNet1"

$routingsubnet = Get-NetworkControllerVirtualSubnet -ConnectionUri $uri -ResourceId “Tenant1_VNet1_RoutingSubnet”
$pool = Get-NetworkControllerGatewayPool -ConnectionUri $uri -ResourceId “Pool1”
$vGateway = New-Object Microsoft.Windows.NetworkController.VirtualGatewayProperties
$vGateway.GatewayPools = @()
$vGateway.GatewayPools += $pool
$vGateway.GatewaySubnets = @()
$vGateway.GatewaySubnets += $routingSubnet
$vGateway.RoutingType = “Dynamic”
$vGateway.NetworkConnections = @()
$vGateway.BgpRouters = @()
New-NetworkControllerVirtualGateway -ConnectionUri $uri -ResourceId “Tenant1_Gateway” -Properties $vGateway

#Integrate VLANs with RAS Gateway
Set-VMNetworkAdapterIsolation –VMName “Tenant1_VM1” -AllowUntaggedTraffic $true -IsolationMode VLAN -DefaultIsolationId 123

$subnet = Get-NetworkControllerVirtualSubnet -ConnectionUri $uri -ResourceId “Tenant1_VNet1_Subnet1”
$nic = New-Object Microsoft.Windows.NetworkController.NetworkInterfaceProperties
$nic.PrivateMacAddress = “001122AABBCC”
$nic.PrivateMacAllcationMethod = “Static”
$nic.DnsSettings = [Microsoft.Windows.NetworkController.NetworkInterfaceDnsSettings]@{DnsServers=@("192.168.0.10", "192.168.0.11")}
$ip = New-Object Microsoft.Windows.NetworkController.NetworkInterfaceIpConfiguration
$ip.ResourceId = “Tenant1_VM1”
$ip.properties = [Microsoft.Windows.NetworkController.NetworkInterfaceIpConfigurationProperties]@{PrivateIPAddress=”192.168.0.4”;PrivateIPAllocationMethod=”Static”}
$ip.properties.Subnet = $subnet
$nic.IPConfigurations = @($ip)
$vnic = New-NetworkControllerNetworkInterface -ConnectionUri $uri -ResourceId “Tenant1_VM1_Nic1” -Properties $nic

$InstanceID = $vnic.InstanceId
#The hardcoded Ids in this section are fixed values and must not change.
$FeatureId = "9940cd46-8b06-43bb-b9d5-93d50381fd56"
$vmNics = Get-VMNetworkAdapter -VMName “Tenant1_VM1”
$CurrentFeature = Get-VMSwitchExtensionPortFeature -FeatureId $FeatureId -VMNetworkAdapter $vmNic
if ($CurrentFeature -eq $null)
{
    $Feature = Get-VMSystemSwitchExtensionFeature -FeatureId $FeatureId
    $Feature.SettingData.ProfileId = "{$InstanceId}"
    $Feature.SettingData.NetCfgInstanceId = "{56785678-a0e5-4a26-bc9b-c0cba27311a3}"
    $Feature.SettingData.CdnLabelString = "TestCdn"
    $Feature.SettingData.CdnLabelId = 1111
    $Feature.SettingData.ProfileName = "Testprofile"
    $Feature.SettingData.VendorId = "{1FA41B39-B444-4E43-B35A-E1F7985FD548}"
    $Feature.SettingData.VendorName = "NetworkController"
    $Feature.SettingData.ProfileData = 1
    Add-VMSwitchExtensionFeature -VMSwitchExtensionFeature  $Feature -VMNetworkAdapter $vmNic
}        
else
{
    $CurrentFeature.SettingData.ProfileId = "{$InstanceId}"
    $CurrentFeature.SettingData.ProfileData = 1
    Set-VMSwitchExtensionPortFeature -VMSwitchExtensionFeature $CurrentFeature  -VMNetworkAdapter $vmNic
}