﻿<#
.SYNOPSIS
Script for setting up SQL failover cluster

.DESCRIPTION
This script will configure failover clustering to support a SQL failover cluster.


#>
param
(
	#root cluster object name
	[Parameter(Mandatory=$true)]
	[string]$clustername,
	
	#root cluster object ip address
	[Parameter(Mandatory=$false)]
    [ValidateScript({ $_ -match [ipaddress] $_})]
	[string]$clusterIP = "0.0.0.0",
	
	#first node in cluster switch
	[switch]$FirstNode,
	
	#debug mode
	[switch]$VerboseMode
);

function Log($out) { 
    $out = [System.DateTime]::Now.ToString("yyyy.MM.dd hh:mm:ss") + " ---- " + $out; 
    Write-Output "$out"; 
}

function Log-Verbose($out) {
	if ($VerboseMode) {
		Log $out;
	}
}
Log-Verbose "Parameters";
Log-Verbose "Cluster Name:`t`t$clustername";
Log-Verbose "Cluster IP:`t`t$clusterIP";
Log-Verbose "First Node Flag:`t$FirstNode";

#Cluster Configuration
if ($FirstNode) {
	if($clusterIP -ne "0.0.0.0") { 
		Log "Creating cluster $clustername on $env:COMPUTERNAME with IP $clusterIP"; 
		New-Cluster -Name $clustername -Node localhost -StaticAddress $clusterIP | Out-Null; 
	} else { 
		Log "Creating cluster $clustername on $env:COMPUTERNAME using DHCP"; 
		New-Cluster -Name $clustername -Node localhost | Out-Null; 
	}
}
else {
	for($i=0;$i -lt 6;$i++) {
		if ($([System.Net.Dns]::GetHostAddresses($clustername))) {
			break;
		}
		Log-Verbose "Waiting for DNS..."; 
		Start-Sleep -s 20
	}

	Log "Adding localhost to $clustername"; 
	Get-Cluster -Name $clustername | Add-ClusterNode -Name localhost; 
}

Log "Validating cluster..."; 
$results = Test-Cluster -WarningAction SilentlyContinue; 
if(Select-String '<div class=3D"error">' $results) { 
	Log "Cluster Validation errors."; 
	Write-Error "Validation failure. See '$results' for details."; 
	exit -1; 
} 

Log "Validation complete." 
